// ignore_for_file: public_member_api_docs

import 'dart:math';

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:healthrian_ble_interface/healthrian_ble_interface.dart';
import 'package:healthrian_ble_support/healthrian_ble_support.dart';

part 'battery_monitor.g.dart';

_$BatteryMonitorCWProxy get BatteryWearEcg =>
    BatteryMonitor(module: BLEManager().wearEcg12.module).copyWith;

_$BatteryMonitorCWProxy get BatteryCardioClick =>
    BatteryMonitor(module: BLEManager().cardioClick.module).copyWith;

_$BatteryMonitorCWProxy get BatteryWearT =>
    BatteryMonitor(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class BatteryMonitor extends StatelessWidget {
  const BatteryMonitor({
    required this.module,
    super.key,
    this.height = 50,
    this.onLoadingWidget,
    this.wrapperWidget,
  });

  final ModuleInterface module;

  final double height;
  final Widget? onLoadingWidget;

  final Widget Function(int)? wrapperWidget;

  @override
  Widget build(BuildContext context) {
    final f = wrapperWidget ??
        (int value) {
          const levels = [
            Icon(
              Icons.battery_full,
              color: Colors.green,
            ),
            Icon(
              Icons.battery_6_bar,
              color: Colors.green,
            ),
            Icon(
              Icons.battery_5_bar,
              color: Colors.green,
            ),
            Icon(
              Icons.battery_4_bar,
              color: Colors.amber,
            ),
            Icon(
              Icons.battery_3_bar,
              color: Colors.amber,
            ),
            Icon(
              Icons.battery_2_bar,
              color: Colors.amber,
            ),
            Icon(
              Icons.battery_1_bar,
              color: Colors.redAccent,
            ),
            Icon(
              Icons.battery_0_bar,
              color: Colors.redAccent,
            ),
          ];

          final index = 7 - ((value - 1) ~/ 12.5);
          return Row(
            children: [
              Transform.rotate(
                angle: pi / 2,
                child: levels[index],
              ),
              Text('$value%'),
            ],
          );
        };

    return Observer(
      key: UniqueKey(),
      builder: (context) {
        return StreamBuilder(
          stream: module.ble.streams.battery,
          builder: (context, snapshot) {
            if (!snapshot.hasData) return onLoadingWidget ?? const SizedBox();
            return f(snapshot.data!);
          },
        );
      },
    );
  }
}
