// ignore_for_file: public_member_api_docs

import 'dart:math';

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:healthrian_ble_interface/healthrian_ble_interface.dart';
import 'package:healthrian_ble_support/healthrian_ble_support.dart';

part 'connection_monitor.g.dart';

_$ConnectionMonitorCWProxy get ConnectionWearEcg =>
    ConnectionMonitor(module: BLEManager().wearEcg12.module).copyWith;

_$ConnectionMonitorCWProxy get ConnectionCardioClick =>
    ConnectionMonitor(module: BLEManager().cardioClick.module).copyWith;

_$ConnectionMonitorCWProxy get ConnectionWearT =>
    ConnectionMonitor(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class ConnectionMonitor extends StatelessWidget {
  const ConnectionMonitor({
    required this.module,
    super.key,
    this.width = 50,
    this.height = 50,
    this.onConnectedWidget,
    this.onDisconnectedWidget,
  });

  final ModuleInterface module;

  final double width;
  final double height;

  final Widget? onConnectedWidget;
  final Widget? onDisconnectedWidget;

  @override
  Widget build(BuildContext context) {
    return Observer(
      key: UniqueKey(),
      builder: (context) {
        return StreamBuilder(
          stream: module.ble.streams.connection,
          builder: (context, snapshot) {
            final connected = snapshot.data ?? false;
            return SizedBox(
              width: width,
              height: height,
              child: connected
                  ? (onConnectedWidget ??
                      Icon(
                        Icons.check,
                        size: min(width, height),
                      ))
                  : (onDisconnectedWidget ??
                      Icon(
                        Icons.close,
                        size: min(width, height),
                      )),
            );
          },
        );
      },
    );
  }
}
