// ignore_for_file: public_member_api_docs

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:healthrian_ble_interface/healthrian_ble_interface.dart';
import 'package:healthrian_ble_support/healthrian_ble_support.dart';
import 'package:healthrian_visualization_support/'
    'healthrian_visualization_support.dart';

part 'ecg_monitor.g.dart';

_$EcgMonitorCWProxy get EcgWearEcg =>
    EcgMonitor(module: BLEManager().wearEcg12.module).copyWith;

_$EcgMonitorCWProxy get EcgCardioClick =>
    EcgMonitor(module: BLEManager().cardioClick.module).copyWith;

_$EcgMonitorCWProxy get EcgWearT =>
    EcgMonitor(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class EcgMonitor extends StatelessWidget {
  const EcgMonitor({
    super.key,
    required this.module,
    this.onLoadingWidget,
    this.width,
    this.height,
    this.color = Colors.black,
    this.measureTime = 5,
    this.wrapperWidget,
    this.pivotColor = Colors.transparent,
    this.pivotStrokeWidth = 0,
    this.strokeWidth = 1.5,
    this.maxY = 2,
    this.minY = -2,
    this.backgroundColor = Colors.transparent,
    this.ignoreChannels = const [],
  });

  final ModuleInterface module;

  final double measureTime;
  final Color color;
  final double? width;
  final double? height;
  final Widget? onLoadingWidget;
  final Color pivotColor;
  final double pivotStrokeWidth;

  final double strokeWidth;
  final double minY;
  final double maxY;

  final Color backgroundColor;

  final List<int> ignoreChannels;

  final Widget Function(List<Widget>)? wrapperWidget;

  @override
  Widget build(BuildContext context) {
    final streams = module.ble.streams;
    return Observer(
      key: UniqueKey(),
      builder: (context) {
        return StreamBuilder(
          stream: streams.plotRTEcg,
          builder: (context, snapshot) {
            if (!snapshot.hasData) {
              return onLoadingWidget ?? const Text('No Stream');
            }
            final list = snapshot.data!.list;
            final pivot = snapshot.data!.pivot;

            final children = [
              for (var i = 0; i < list.length; i++)
                if (!ignoreChannels.contains(i))
                  SizedBox(
                    width: width ?? MediaQuery.of(context).size.width,
                    height: height ?? 50,
                    child: ChannelWithPivot(
                      ecgSequence: list[i],
                      color: color,
                      measureTime: measureTime,
                      pivot: pivot,
                      pivotColor: pivotColor,
                      pivotStrokeWidth: pivotStrokeWidth,
                      backgroundColor: backgroundColor,
                      samplingRate: module.ecg?.samplingRate ?? 250,
                      maxY: maxY,
                      minY: minY,
                      strokeWidth: strokeWidth,
                    ),
                  ),
            ];

            if (wrapperWidget == null) {
              return Column(
                children: children,
              );
            }
            return wrapperWidget!(children);
          },
        );
      },
    );
  }
}
