// ignore_for_file: public_member_api_docs

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:healthrian_ble_interface/healthrian_ble_interface.dart';
import 'package:healthrian_ble_support/healthrian_ble_support.dart';
import 'package:healthrian_visualization_support/healthrian_visualization_support.dart';

part 'leadstate_monitor.g.dart';

_$LeadstateMonitorCWProxy get LeadstateWearEcg =>
    LeadstateMonitor(module: BLEManager().wearEcg12.module).copyWith;

_$LeadstateMonitorCWProxy get LeadstateCardioClick =>
    LeadstateMonitor(module: BLEManager().cardioClick.module).copyWith;

_$LeadstateMonitorCWProxy get LeadstateWearT =>
    LeadstateMonitor(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class LeadstateMonitor extends StatelessWidget {
  const LeadstateMonitor({
    required this.module,
    super.key,
    this.width = 300,
    this.height = 150,
    this.onLoadingWidget,
    this.wrapperWidget,
  });

  final ModuleInterface module;

  final double width;
  final double height;
  final Widget? onLoadingWidget;

  final Widget Function(List<bool>)? wrapperWidget;

  @override
  Widget build(BuildContext context) {
    final f = wrapperWidget ??
        (List<bool> leadstate) {
          return CustomPaint(
            size: Size(width, height),
            painter: DrawLead(leadstate: leadstate),
          );
        };

    return Observer(
      key: UniqueKey(),
      builder: (context) {
        return StreamBuilder(
          stream: module.ble.streams.leadStateEcg,
          builder: (context, snapshot) {
            if (!snapshot.hasData) return onLoadingWidget ?? f([]);
            final leadstate = snapshot.data?.list ?? [];
            return f(leadstate);
          },
        );
      },
    );
  }
}
