// ignore_for_file: public_member_api_docs, lines_longer_than_80_chars

import 'package:flutter/material.dart';
import 'package:healthrian_ble_support/healthrian_ble_support.dart';

class BLEList extends StatefulWidget {
  const BLEList({
    super.key,
  });

  @override
  State<BLEList> createState() => _BLEListState();
}

class _BLEListState extends State<BLEList> {
  Future<void> _rescan() async {
    await BLEManager().raw.stopScan();

    setState(() {});

    await BLEManager().raw.startScan(
          timeout: const Duration(seconds: 4),
        );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: StreamBuilder(
        stream: BLEManager().raw.scanResults,
        builder: (context, snapshotScanned) {
          if (!snapshotScanned.hasData) {
            return const SizedBox();
          }

          return FutureBuilder(
            future: BLEManager().raw.bondedDevices,
            builder: (context, snapshot) {
              if (!snapshot.hasData) return const SizedBox();
              final bonded = snapshot.data!;
              return FutureBuilder(
                future: BLEManager().raw.connectedDevices,
                builder: (context, snapshot) {
                  if (!snapshot.hasData) return const SizedBox();

                  final connected = snapshot.data!;

                  final scanned = snapshotScanned.data!;
                  // ..sort(
                  //   (a, b) => b.rssi - a.rssi,
                  // );

                  final paired = scanned.where(
                    (element) => bonded.contains(element.device),
                  );
                  final unpaired = scanned
                      .where(
                        (element) => !bonded.contains(element.device),
                      )
                      .toList(growable: false)
                    ..sort((a, b) => b.rssi - a.rssi);

                  final sorted = paired.followedBy(unpaired).toList(
                        growable: false,
                      );

                  return ListView.separated(
                    itemBuilder: (_, index) {
                      if (index < connected.length) {
                        return ListTile(
                          onTap: () async {
                            await connected[index].disconnect();
                            await _rescan();
                            setState(() {});
                          },
                          title: Text(
                            connected[index].name,
                          ),
                          subtitle: const Text('Connected'),
                        );
                      }

                      final sIndex = index - connected.length;
                      return ListTile(
                        onTap: () async {
                          await sorted[sIndex].device.pair();
                          await sorted[sIndex].device.connect();
                          await _rescan();
                          setState(() {});
                        },
                        title: Text(sorted[sIndex].device.name),
                        subtitle: Text('rssi: ${sorted[sIndex].rssi}'),
                        trailing: bonded.contains(sorted[sIndex].device) ? const Text('Paired') : null,
                      );
                    },
                    separatorBuilder: (_, __) {
                      return const Divider();
                    },
                    itemCount: connected.length + sorted.length,
                  );
                },
              );
            },
          );
        },
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: _rescan,
      ),
    );
  }
}
